function adcig1(min_g,n_g,v1,v2,zwb)
%function adcig1(min_g,n_g,v1,v2,zwb)
%	
%	function to compute the moveout of a water-bottom multiple from
%	a flat water-bottom in angle-domain common-image gathers
%
%	Input:
%		min_g: minimum aperture angle (degrees)
%		n_g: number of aperture angles
%		v1: velocity of the water (m/s)
%		v2: velocity of the sediments (m/s)
%		zwb: depth of the flat water-bottom (m)
%
%	Example:
%		adcig1(0,21,1500,2500,150)
%		adcig1(0,75,1500,2500,500)
%
	
	% compute rho
	rho=v2/v1
	rho1=1.0/rho

	% set aperture angles
	gamma=linspace(min_g,45,n_g);
	gamma=deg2rad(gamma);

	% compute image depths
	zi=zwb*(1+(cos(gamma).*(rho^2-tan(gamma).^2*(1-rho^2)))./(sqrt(rho^2-sin(gamma).^2)));

	% Biondo's tangent squared
	z0=zwb*(1.+rho)
	%deltaRMO=z0*(rho-1)*(tan(gamma)).^2;
	deltaRMO=zwb*(1-rho)*(tan(gamma)).^2;
	zi1=z0-deltaRMO;
	
	% write zi to an ascii file
	fid=fopen('adcig1.ascii','w');
	for i=1:length(gamma)
		fprintf(fid,'%8.4f\n',zi(i));
	end
	fclose(fid);
	gamma=rad2deg(gamma);

	% plot the moveout curve
	figure;
	plot(gamma,zi,'LineWidth',2);
	axis ij;
	%axis([0 45 180 600]);
	grid on;
	xlabel('Aperture angle (degrees)');
	ylabel('Depth (m)');
	title('Water-bottom multiple. Flat water-bottom');
	hold on;
	plot(gamma,zi1,'r','LineWidth',2);
	legend('New','Old');
	figure;
	plot(gamma,zi1,'r','LineWidth',2);
	axis ij;
	%axis([0 45 180 600]);
	grid on;
	xlabel('Aperture angle (degrees)');
	ylabel('Depth (m)');
	title('Water-bottom multiple. Flat water-bottom');
return
