!! Modified to pass two additional parameters: rho and mine, to the tansimp_init
!! subroutine. The mine flag is to choose between tangent squared (mine==0)
!! and the new moveout (mine==1). GA
program ART

  use sep
  
  ! time domain approach with Huber function
  use tansimp_mod
  use huber_solver
  use morethuente_mod
  use huber_mod
  use lbfgs_mod
  use ddot_mod
 
  implicit none

  integer                          :: nk, nt, nx, nq, iadj, n2, stat
  real                             :: eps,gamd,gamx
  real                             :: o1,d1,o2,d2,o3,d3,x0,dx,q0,dq,rho
  real, dimension (:), allocatable :: data, model, recons
  logical                          :: adj
  integer                          :: n3, j, verb, niter, l1, mine

  call sep_init ()                 ! start processing parameters
  
  call from_history ("n3", n3)
  call from_history ("o3", o3)
  call from_history ("d3", d3)
  call from_history (nt  , n2)
  call from_history ("o2", o2)
  call from_history ("d2", d2)
  call from_either  ("nx", nx, n2)
  call from_either  ("nq", nq, 2*nx)
  call from_history ("o1", o1)
  call from_history ("d1", d1)
  call from_either  ("q0", q0) 
  call from_either  ("dq", dq)  
  call from_either  ("x0", x0, o2) 
  call from_either  ("dx", dx, d2) 
  call from_param   ("verb",verb,0)  
  call from_param   ("adj",iadj,1) ; adj = (iadj == 1)
  call from_param   ("gamd",gamd,1.)
  call from_param   ("gamx",gamx,0.)
  call from_param   ("eps",eps,0.)
  call from_param   ("rho",rho,1.)
  call from_param   ("l1",l1,1)
  call from_param   ("niter",niter,10)
  call from_param   ("mine",mine,0)

  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>>'
  if (verb>0) write(0,*) '>> ART in the Time'
  if (verb>0) write(0,*) '>> ART - '
  if (verb>0) write(0,*) '>> ART - '
  if (verb>0) write(0,*) '>> ART - parameters:'
  if (verb>0) write(0,*) '>> ART - '
  if (verb>0) write(0,*) '>> ART - '
  if (verb>0) write(0,*) '>> ART - q0  =:',q0
  if (verb>0) write(0,*) '>> ART - nq  =:',nq
  if (verb>0) write(0,*) '>> ART - dq  =:',dq
  if (verb>0) write(0,*) '>> ART - verb=:',verb
  if (verb>0) write(0,*) '>> ART - gamd  =:',gamd
  if (verb>0) write(0,*) '>> ART - gamx  =:',gamx
  if (verb>0) write(0,*) '>> ART - niter =:',niter
  if (verb>0) write(0,*) '>> ART - eps   =:',eps
  if (verb>0) write(0,*) '>> ART - l1    =:',l1
  if (verb>0) write(0,*) '>> ART - '
  if (verb>0) write(0,*) '>> ART - '     

  if (adj) then 
     call to_history ("label2"," q ")
     n2 = nq; o2 = q0; d2 = dq
  else
     call to_history ("label2","distance")
     n2 = nx; o2 = x0; d2 = dx
  end if
  
  call to_history ("o2", o2)
  call to_history ("d2", d2)
  call to_history ("n2", n2)

  if (exist_file ("recons")) then
     call to_history ("o1", o1,"recons")
     call to_history ("d1", d1,"recons")
     call to_history ("n1", nt,"recons")
     call to_history ("o2", x0,"recons")
     call to_history ("d2", dx,"recons")
     call to_history ("n2", nx,"recons")
     call to_history ("o3", o3,"recons")
     call to_history ("d3", d3,"recons")
     call to_history ("label2","distance(km)","recons")
     allocate(recons(nt*nx))
  end if
    
  call sep_close()                ! end processing parameters
  
  allocate (data(nt*nx), model(nt*nq))
  
  if (verb>0) write(0,*) '               ART >> Time Domain - Quasi-Newton / L-BFGS '
  call tansimp_init(o1,d1,x0,dx,q0,dq,nt,nx,nq,rho,mine,verb)
  call lbfgs_init(5,nt*nq,niter,verb)
  call MTlinesearch_init(0.9,0.0001,verb)
  call huber_init_out(gamd,gamx,eps,l1,verb)
  
  do j=1,n3

     data=0.
     model=0.
     
     if (verb>0) write(0,*) '>>>> ART -- GATHER #',j,'/',n3 

     if (adj) then  
        
        call sep_read(data)
                 
        call huber_init_in(data)
        call hub (tansimp_lop,lbfgs,MTlinesearch,huberfct,hubergdt,model,niter,verb)
        call huber_close()
        
        call sep_write (model)
        
        if (exist_file("recons")) then
           
           stat = tansimp_lop(.false.,.false.,model,data)
           
           call sep_write(data,"recons")
        
        end if
        
     else
        
        call sep_read (model)
                
        stat = tansimp_lop (adj,.false.,model,data)

        call sep_write (data)
        
     end if
  
  end do
  
  deallocate (model,data)
  if (exist_file ("recons")) deallocate(recons)
  
  call exit (0)
  
end program ART
