module morethuente_mod
  
  use ddot_mod
  
  implicit none
  
  real                        :: num
  real                        :: mum
  integer, private            :: verb
  logical, parameter, private :: T = .true., F = .false.
  
contains
  
  subroutine MTlinesearch_init(n,m,verb_in)
    real   ::n,m
    integer::verb_in
    
    num = n
    mum = m
    verb= verb_in
    
  end subroutine MTlinesearch_init
  
  subroutine psifct(oper,fonction,xk,rk,ak,psi,f0,g0)
    
    interface
       function oper( adj, add, x, dat) result(stat)
         integer                   :: stat
         logical, intent(in)       :: adj, add
         real, dimension(: )       :: x, dat
       end function oper
       
       function fonction(oper,x,fct) result (stat)
         
         interface
            function oper( adj, add, x, dat) result(stat)
	      integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface
         
         integer             :: stat
         real,dimension(:)   :: x
         real                :: fct
         
       end function fonction
       
    end interface
    
    real, dimension(:), intent(in)  :: xk  
    real, dimension(:), intent(in)  :: rk   
    real              , intent(in ) :: ak, g0, f0
    real              , intent(out) :: psi
    real                            :: tmp1, tmp4
    integer                         :: stat1
    
    stat1 = fonction(oper,xk+ak*rk,tmp1)
    tmp4  = mum*ak*g0
    
    psi   = tmp1 - f0 - tmp4
    
  end subroutine psifct
  
  subroutine psigdt(oper,gradient,xk,rk,ak,psi,g0)
    
    interface
       function oper( adj, add, x, dat) result(stat)
         integer                   :: stat
         logical, intent(in)       :: adj, add
         real, dimension(: )       :: x, dat
       end function oper
       
       function gradient(oper,x,g) result (stat)
         
         interface
            function oper( adj, add, x, dat) result(stat)
	      integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface
         
         integer             :: stat
         real,dimension(:)   :: x,g
         
       end function gradient
    end interface
    
    real, dimension(:), intent(in)  :: xk  
    real, dimension(:), intent(in)  :: rk       
    real              , intent(in ) :: ak, g0
    real              , intent(out) :: psi
    integer                         :: stat1
    real,dimension(:),pointer       :: tmp1
    
    allocate(tmp1(size(xk)))
    
    stat1 = gradient(oper,xk+ak*rk,tmp1)
    
    psi = ddot(tmp1,rk) - mum * g0
    
    deallocate(tmp1)
    
  end subroutine psigdt
  
  subroutine setat(at,al,au,ft,fl,fu,gt,gl,gu,brackt,apmin,apmax,info)
    
    real                          :: at,al,au,apmin,apmax
    real                          :: atp,ac,aq
    real                          :: ft,fl,fu
    real                          :: gt,gl,gu
    real                          :: gamma,P,Q,R,S,sgnd,theta
    logical                       :: brackt, bound
    integer                       :: info
    
    if ((brackt.and.(at.le.min(al,au).or.at.ge.max(al,au))) .or. &
    gl*(at-al).ge.0.0) then
       write(0,*) 'fucking problem in setat input'
    end if
    
    sgnd = gt*(gl/abs(gl))
    
    ! FIRST CASE. A HIGHER FUNCTION VALUE. THE MINIMUM IS BRACKETED.
    ! IF THE CUBIC STEP IS CLOSER TO STX THAN THE QUADRATIC STEP,
    ! THE CUBIC STEP IS TAKEN, ELSE THE AVERAGE OF THE CUBIC AND
    ! QUADRATIC STEPS IS TAKEN
    
    if (ft .GT. fl) then
       
       info = 1
       bound = .TRUE.
       theta = 3*(fl - ft)/(at - al) + gl + gt
       S = MAX(ABS(theta),ABS(gl),ABS(gt))
       gamma = S*SQRT((theta/S)**2 - (gl/S)*(gt/S))
       
       if (at .LT. al) then
          gamma = -gamma
       end if
       
       P = (gamma - gl) + theta
       Q = ((gamma - gl) + gamma) + gt
       R = P/Q
       ac = al + R*(at - al)
       aq = al + ((gl/((fl-ft)/(at-al)+gl))/2)*(at - al)
       
       if (ABS(ac-al) .LT. ABS(aq-al)) then 
          atp = ac
       else
          atp = ac + (aq - ac)/2
       end if
       
       brackt = .TRUE.
       
       !
       ! SECOND CASE. A LOWER FUNCTION VALUE AND DERIVATIVES OF
       ! OPPOSITE SIGN. THE MINIMUM IS BRACKETED. IF THE CUBIC
       ! STEP IS CLOSER TO al THAN THE QUADRATIC (SECANT) STEP,
       ! THE CUBIC STEP IS TAKEN, ELSE THE QUADRATIC STEP IS TAKEN.
       !
       
    else if (sgnd .LT. 0.0) then
       
       info = 2
       bound = .FALSE.
       theta = 3*(fl - ft)/(at - al) + gl + gt
       S = MAX(ABS(theta),ABS(gl),ABS(gt))
       gamma = S*SQRT((theta/S)**2 - (gl/S)*(gt/S))
       
       if (at .GT. al) then
          gamma = -gamma
       end if
       
       P = (gamma - gt) + theta
       Q = ((gamma - gt) + gamma) + gl
       R = P/Q
       ac = at + R*(al - at)
       aq = at + (gt/(gt-gl))*(al - at)
       
       if (ABS(ac-at) .GT. ABS(aq-at)) then 
          atp = ac
       else
          atp = aq
       end if
       
       brackt = .TRUE.
       !
       ! THIRD CASE. A LOWER FUNCTION VALUE, DERIVATIVES OF THE
       ! SAME SIGN, AND THE MAGNITUDE OF THE DERIVATIVE DECREASES.
       ! THE CUBIC STEP IS ONLY USED IF THE CUBIC TENDS TO INFINITY
       ! IN THE DIRECTION OF THE STEP OR IF THE MINIMUM OF THE CUBIC
       ! IS BEYOND at. OTHERWISE THE CUBIC STEP IS DEFINED TO BE
       ! EITHER apmin OR apmax. THE QUADRATIC (SECANT) STEP IS ALSO
       ! COMPUTED AND IF THE MINIMUM IS BRACKETED THEN THE THE STEP
       ! CLOSEST TO al IS TAKEN, ELSE THE STEP FARTHEST AWAY IS TAKEN.
       !
       
    else if (ABS(gt) .LT. ABS(gl)) then
       
       info = 3
       bound = .TRUE.
       theta = 3*(fl - ft)/(at - al) + gl + gt
       S = MAX(ABS(theta),ABS(gl),ABS(gt))
       !
       ! THE CASE gamma = 0 ONLY ARISES IF THE CUBIC DOES NOT TEND
       ! TO INFINITY IN THE DIRECTION OF THE STEP.
       !
       gamma = S*SQRT(MAX(0.0,(theta/S)**2 - (gl/S)*(gt/S)))
       
       if (at .GT. al) then
          gamma = -gamma
       end if
       
       P = (gamma - gt) + theta
       Q = (gamma + (gl - gt)) + gamma
       R = P/Q
       
       if (R .LT. 0.0 .AND. gamma .NE. 0.0) then 
          ac = at + R*(al - at)
       else if (at .GT. al) then
          ac = apmax
       else
          ac = apmin
       end if
       
       aq = at + (gt/(gt-gl))*(al - at)
       
       if (brackt) then
          if (ABS(at-ac) .LT. ABS(at-aq)) then 
             atp = ac
          else
             atp = aq
          end if
       else
          if (ABS(at-ac) .GT. ABS(at-aq)) then 
             atp = ac
          else
             atp = aq 
          end if
       end if
       !
       ! FOURTH CASE. A LOWER FUNCTION VALUE, DERIVATIVES OF THE
       ! SAME SIGN, AND THE MAGNITUDE OF THE DERIVATIVE DOES
       ! NOT DECREASE. IF THE MINIMUM IS NOT BRACKETED, THE STEP
       ! IS EITHER apmin OR apmax, ELSE THE CUBIC STEP IS TAKEN.
       !
       
    else
       
       info = 4
       bound = .FALSE.
       
       if (brackt) then
          theta = 3*(ft - fu)/(au - at) + gu + gt
          S = MAX(ABS(theta),ABS(gu),ABS(gt))
          gamma = S*SQRT((theta/S)**2 - (gu/S)*(gt/S))
          
          if (at .GT. au) then
             gamma = -gamma
          end if
          
          P = (gamma - gt) + theta 
          Q = ((gamma - gt) + gamma) + gu
          R = P/Q
          ac = at + R*(au - at)
          atp = ac
          
       else if (at .GT. al) then
          atp = apmax
       else
          atp = apmin
       end if
    end if
    !
    ! UPDATE THE INTERVAL OF UNCERTAINTY. THIS UPDATE DOES NOT
    ! DEPEND ON THE NEW STEP OR THE CASE ANALYSIS ABOVE.
    !
    
    if (ft .GT. fl) then
       au = at
       fu = ft
       gu = gt
    else
       if (SGND .LT. 0.0) then
          au = al
          fu = fl
          gu = gl
       end if
       al = at
       fl = ft
       gl = gt
    end if
    !
    ! COMPUTE THE NEW STEP AND SAFEGUARD IT.
    !
    atp = MIN(apmax,atp)
    atp = MAX(apmin,atp)
    at = atp
    
    if (brackt .AND. bound) then
       if (au .GT. al) then
          at = MIN(al+0.66*(au-al),at)
       else
          at = MAX(al+0.66*(au-al),at)
       end if
    end if
    !
    ! LAST LINE OF SUBROUTINE setat.
    !
    
    
  end subroutine setat
  
  function MTlinesearch(oper,fonction,gradient,xk,rk,ak,info) result (stat)
    
    interface
       function oper( adj, add, x, dat) result(stat)
         integer                   :: stat
         logical, intent(in)       :: adj, add
         real, dimension(: )       :: x, dat
       end function oper
       
       function fonction(oper,x,fct) result (stat)
         
         interface
            function oper( adj, add, x, dat) result(stat)
              integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface
         
         integer             :: stat
         real,dimension(:)   :: x
         real                :: fct
         
       end function fonction
       
       function gradient(oper,x,g) result (stat)
         
         interface
            function oper( adj, add, x, dat) result(stat)
              integer             :: stat
              logical, intent(in) :: adj, add
              real, dimension(: ) :: x, dat
            end function oper
         end interface
         
         integer             :: stat
         real,dimension(:)   :: x,g
         
       end function gradient
       
    end interface
    
    real, dimension(:),intent(in) :: xk   ! solution
    real, dimension(:),intent(in) :: rk   ! gradient
    real              ,intent(out):: ak   ! step
    real                          :: al   ! lower bound of the interval
    real                          :: au   !  uper bound of the interval
    real                          :: at   ! update for the new bound
    real                          :: amin, amax, apmax, apmin, xtol
    real                          :: width,width1,atrapf, tmp5
    real                          :: ft, fu, fl ,gt ,gu, gl, f0, g0
    real                          :: phip
    integer                       :: stat
    integer                       :: modified
    real,dimension(:),pointer     :: tmp,tmp1,tmp2,tmp3
    integer                       :: count, limit, infoc, info
    logical                       :: brackt
    
    allocate(tmp(size(xk))) 
    allocate(tmp1(size(xk))) 
    allocate(tmp2(size(xk))) 
    allocate(tmp3(size(xk)))
    
    ! ideally, apmin and apmax are 10e20 10e-20 
    
    apmin = 0.000000000000001 
    apmax = 10000000000000000.
    
    ! xtol should be the machine precision
    
    xtol = 0.000000000000001 
    
    infoc = 1
    info = 0
    modified = 0
    count  = 0
    limit = 10
    atrapf = 4.
    brackt = .false.
    
    stat = fonction(oper,xk,f0)     ! useful for the Wolfe test
    stat = gradient(oper,xk,tmp1)
    g0 = ddot(tmp1,rk)
    
    fu = f0
    gu = g0
    fl = f0
    gl = g0
    au = 0.0
    al = 0.0
    at = 1.0
    
    width = (apmax - apmin)
    width1= width/0.5
    
    do while (info.eq.0)
       
       count = count + 1
       
       ! Set the minimum and maximum steps to correspond to
       ! the present interval of uncertainty
       
       if (brackt) then
          amin = min(al,au)
          amax = max(al,au)
       else
          amin = al
          amax = at + atrapf*(at-al)
       end if
       
       ! Force the step to be within the bounds 
       ! apmax and apmin
       
       at = max(at,apmin)  
       at = min(at,apmax)
       
       ! if an unusual terminationis to occur, then let
       ! at be the lowest point abtained so far
       
       if ((brackt.and.(at.le.amin .or. at.ge.amax)) .or. &
       count.ge.limit-1 .or. infoc.eq.0 .or. &
       (brackt .and. (amax-amin .le. xtol*amax))) then
          at = at
       end if
       
       ! check if we have the WOLFE convergence conditions
       
       stat = fonction(oper,xk+at*rk,ft)
       stat = gradient(oper,xk+at*rk,tmp1)
       gt = ddot(tmp1,rk)
       tmp5 = f0 + mum*at*g0
       
       ! Test for convergence and other stuffs to check out
       
       if ((brackt.and.(at.le.amin .or. at.ge.amax)) .or. &
       infoc.eq.0) then
          info = 6
          write(0,*) ' info = 6'
       end if
       
       if (at.eq.apmax .and. ft.le.tmp5 .and. gt.le.mum*g0) then
          info = 5
          write(0,*) ' info = 5'
       end if
       
       if (at.eq.apmin .and. ft.gt.tmp5 .and. gt.ge.mum*g0) then
          info = 4
          write(0,*) ' info = 4'
       end if
       
       if (count.ge.limit) then
          info = 3
          write(0,*) ' info = 3'
       end if
       
       if (brackt .and. amax-amin .le. xtol*amax) then
          info = 2
          write(0,*) ' info = 2'
       end if
       
       if ((ft.le.tmp5).and.(abs(gt).le.num*abs(g0))) then
          info = 1
       end if
       
       if (info.eq.0) then
          
          if (modified.eq.0) then
             
             call psifct(oper,fonction,xk,rk,at,ft,f0,g0) 
             stat = gradient(oper,xk+at*rk,tmp1)
             phip = ddot(tmp1,rk)
             
             if((phip.gt.0).and.(ft.le.0)) then
                modified = 1
             end if
             
          end if
          
          if (modified.eq.0) then
             
             call psifct(oper,fonction,xk,rk,al,fl,f0,g0)
             call psifct(oper,fonction,xk,rk,au,fu,f0,g0)
             call psigdt(oper,gradient,xk,rk,at,gt,g0)
             call psigdt(oper,gradient,xk,rk,al,gl,g0)
             call psigdt(oper,gradient,xk,rk,au,gu,g0)        
             
          else
             
             stat = fonction(oper,xk+at*rk,ft)
             stat = fonction(oper,xk+al*rk,fl)
             stat = fonction(oper,xk+au*rk,fu)
             stat = gradient(oper,xk+at*rk,tmp1)
             stat = gradient(oper,xk+al*rk,tmp2)
             stat = gradient(oper,xk+au*rk,tmp3)
             gt = ddot(tmp1,rk)
             gl = ddot(tmp2,rk)
             gu = ddot(tmp3,rk)
             
          end if
          
          call setat(at,al,au,ft,fl,fu,gt,gl,gu,brackt,amin,amax,infoc)
          
          if (brackt) then
             if (abs(au-al) .ge.0.66*width) then
                at = al + 0.55*(au-al)
             end if
             width1= width
             width = abs(au-al)
          end if
          
       end if
       
    end do
    
    ak = at
    
    if (ak.ne.1.0) then
       if (verb>1) write(0,*) 'Final step =',ak
    end if
    
    if (verb>1) write(0,*) 'function = ',ft
    
    deallocate(tmp,tmp1,tmp2,tmp3)
    stat = 0
    
  end function MTlinesearch
  
end module morethuente_mod

  
